# {{ workcell_description }}
#
# Generated by: madsci new workcell
# Template version: {{ template_version }}

name: {{ workcell_name }}
description: "{{ workcell_description }}"

# Workcell Manager Configuration
workcell_manager:
  url: "http://localhost:8005/"

# Nodes in this workcell
# Add your nodes here with their URLs
nodes:
  # example_node:
  #   url: "http://localhost:2000/"
  #   description: "Example node"

# Workflows available in this workcell
# Reference workflow files or inline definitions
workflows: []
  # - path: "./workflows/my_workflow.workflow.yaml"

# Metadata
metadata:
  version: "1.0.0"
  tags:
    - generated
