# {{ workflow_description }}
#
# Generated by: madsci new workflow --template workflow/basic
# Template version: {{ template_version }}

name: {{ workflow_name }}
description: "{{ workflow_description }}"

# Workflow parameters (passed in when starting the workflow)
parameters:
  - name: message
    type: string
    description: "Message to pass to the action"
    default: "Hello from {{ workflow_name }}!"

# Workflow steps
steps:
  - name: step_1
    description: "Execute {{ action_name }} on {{ node_name }}"
    node: {{ node_name }}
    action: {{ action_name }}
    args:
      message: "${{ '{' }}parameters.message{{ '}' }}"

# Metadata
metadata:
  author: "MADSci User"
  version: "1.0.0"
  tags:
    - generated
    - basic
