[project]
name = "{{ lab_name }}"
version = "0.1.0"
description = "{{ lab_description }}"
requires-python = ">=3.10"
dependencies = [
    "madsci-client",
    "madsci-squid",
    "python-dotenv",
]

[project.optional-dependencies]
dev = [
    "pytest",
    "ruff",
]

[tool.ruff]
line-length = 120
target-version = "py310"

[tool.ruff.lint]
select = ["E", "F", "I", "W"]
