# Python
__pycache__/
*.py[cod]
*$py.class
*.so
*.egg-info/
dist/
build/
*.egg

# Virtual environments
.venv/
venv/
ENV/

# IDE
.vscode/
.idea/
*.swp
*.swo
*~

# OS
.DS_Store
Thumbs.db

# Environment files -- .env holds secrets and is gitignored
# Default configuration lives in settings.yaml (version-controlled)
.env
.env.local
.env.*.local

# MADSci
.madsci/
*.log

# Data
data/
