# Python
__pycache__/
*.py[cod]
*.egg-info/
dist/
build/
.venv/

# Environment -- .env holds secrets and is gitignored
# Default configuration lives in settings.yaml (version-controlled)
.env
.env.local

# Data
data/
backups/
logs/

# Docker
docker-compose.override.yml
