# {{ workflow_description }}
#
# Generated by: madsci new workflow --template workflow/multi_step
# Template version: {{ template_version }}

# Workflow name (derived from workflow_name)
name: {{ workflow_name | replace('_', ' ') | title }}
description: "{{ workflow_description }}"

# Metadata about this workflow
metadata:
  author: "MADSci User"
  version: "1.0.0"
  tags:
    - generated
    - multi-step

# Workflow parameters (passed in when starting the workflow)
# Add any workflow-level parameters your steps need here
parameters: {}

# Workflow steps - executed sequentially
steps:
  # Step 1: Execute the first action on the first node
  - name: step_1
    description: "Execute {{ node_1_action }} on {{ node_1_name }}"
    node: {{ node_1_name }}
    action: {{ node_1_action }}
    args:
      sample_id: "sample_001"

  # Step 2: Execute an action on the second node
  - name: step_2
    description: "Execute {{ node_2_action }} on {{ node_2_name }}"
    node: {{ node_2_name }}
    action: {{ node_2_action }}
    args: {}

  # Step 3: Return to the first node for a follow-up action
  - name: step_3
    description: "Follow-up action on {{ node_1_name }}"
    node: {{ node_1_name }}
    action: report
    args: {}
