# Settings for {{ lab_name }}
# Generated by: madsci new lab --template minimal
#
# This file contains default configuration values and is safe to commit to git.
# Override any value with environment variables (prefixed, e.g. EVENT_SERVER_URL)
# or a .env file (gitignored) for secrets and environment-specific overrides.
# See Configuration.md for all available settings.

# Lab Manager
lab_server_url: http://localhost:8000/

# Event Manager
event_server_url: http://localhost:8001/

# Experiment Manager
experiment_server_url: http://localhost:8002/

# Resource Manager
resource_server_url: http://localhost:8003/

# Data Manager
data_server_url: http://localhost:8004/

# Workcell Manager
workcell_server_url: http://localhost:8005/

# Location Manager
location_server_url: http://localhost:8006/
