# Example workflow for {{ lab_name }}
#
# Generated by: madsci new lab --template standard

name: example_workflow
flowdef:
  - name: step_1
    node: example_node
    action: example_action
    args: {}
    checks: null
    comment: "Replace with your actual workflow steps"
